/*
 * Copyright (c) 1983 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the University of
 *	California, Berkeley and its contributors.
 * 4. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/* Reworked to GNU style by Ian Lance Taylor, ian@airs.com, August 93.  */
/* Reworked to Plan9 style by arisawa@aichi-u.ac.jp Nov 1998*/

/*
 * uudecode [file ...]
 *
 * create the specified file, decoding as you go.
 * used with uuencode.
 */
#include "plan9.h"

int
main (argc, argv)
     int argc;
     char **argv;
{
  int opt;
  int rval;

 	argv0 = argv[0];
	ARGBEGIN{
	case 'h':
	  printf ("Decode a file created by uuencode\n");
	  usage (stdout, 0);
	case 'v':
	  printf ("%s\n", version);
	  exit (0);
	case 0:
	  break;
	default:
	  usage (stderr, 1);
	}ARGEND

  if (*argv == 0)
    rval = decode ((char *) "stdin");
  else
    {
      char *filename;

      rval = 0;
      do
	{
	  if (freopen (*argv, "r", stdin) != NULL)
	    rval |= decode (*argv);
	  else
	    {
	      fprintf (stderr, "%s:", argv0);
	      perror (*argv);
	      rval = 1;
	    }
	  argv++;
	}
      while (*argv);
    }

  exit (rval);
}

#define	DEC(c)	(((c) - ' ') & 077) /* single character decode */

static int
decode (filename)
     char *filename;
{
  struct passwd *pw;
  register int n;
  register char ch, *p;
  int mode, n1;
  char buf[2 * BUFSIZ];
  char *outname;

  /* search for header line */
  do
    {
      if (fgets (buf, sizeof (buf), stdin) == NULL)
	{
	  fprintf (stderr,
		  "%s:%s: no \"begin\" line\n", argv0, filename);
	  return 1;
	}
    }
  while (strncmp (buf, "begin ", 6) != 0);

  sscanf (buf, "begin %o %s", &mode, buf);

  /* handle ~user/file format */
  if (buf[0] != '~')
    outname = buf;
  else
    {
      fprintf (stderr, "%s:%s:", argv0, outname);
      perror (filename);
      return 1;
    }

  /* create output file, set mode */
  if (freopen (outname, "w", stdout) == NULL
      || fchmod (fileno (stdout),
		 mode & 0644))
    {
      fprintf (stderr, "%s:%s:", argv0, outname);
      perror (filename);
      return 1;
    }

  /* for each input line */
  while (1)
    {
      if (fgets (buf, sizeof(buf), stdin) == NULL)
	{
	  fprintf (stderr, "%s:%s: short file.\n", argv0,
		   filename);
	  return 1;
	}
      p = buf;
      /*
       * `n' is used to avoid writing out all the characters
       * at the end of the file.
       */
      n = DEC (*p);
      if (n <= 0)
	break;
      for (++p; n > 0; p += 4, n -= 3)
	{
	  if (n >= 3)
	    {
	      ch = DEC (p[0]) << 2 | DEC (p[1]) >> 4;
	      putchar (ch);
	      ch = DEC (p[1]) << 4 | DEC (p[2]) >> 2;
	      putchar (ch);
	      ch = DEC (p[2]) << 6 | DEC (p[3]);
	      putchar (ch);
	    }
	  else
	    {
	      if (n >= 1)
		{
		  ch = DEC (p[0]) << 2 | DEC (p[1]) >> 4;
		  putchar (ch);
		}
	      if (n >= 2)
		{
		  ch = DEC (p[1]) << 4 | DEC (p[2]) >> 2;
		  putchar (ch);
		}
	    }
	}
    }

  if (fgets (buf, sizeof(buf), stdin) == NULL
      || strcmp (buf, "end\n"))
    {
      fprintf (stderr, "%s:%s: no \"end\" line.\n", argv0,
	       filename);
      return 1;
    }

  return 0;
}

static void
usage (f, status)
     FILE *f;
     int status;
{
  fprintf (f, "usage: %s [file ...]\n", argv0);
  exit (status);
}
